@@ -1,5 +1,7 @@
 package Parse::PMFile;
 
+sub __clean_eval { eval $_[0] } # needs to be here (RT#101273)
+
 use strict;
 use warnings;
 use Safe;
@@ -8,7 +10,7 @@ use Dumpvalue;
 use version ();
 use File::Spec ();
 
-our $VERSION = '0.29';
+our $VERSION = '0.36';
 our $VERBOSE = 0;
 our $ALLOW_DEV_VERSION = 0;
 our $FORK = 0;
@@ -93,7 +95,6 @@ sub parse {
 
         my $pp = $ppp->{$package};
         if ($pp->{version} && $pp->{version} =~ /^\{.*\}$/) { # JSON parser error
-            my $dont_delete;
             my $err = JSON::PP::decode_json($pp->{version});
             if ($err->{x_normalize}) {
                 $errors{$package} = {
@@ -101,8 +102,8 @@ sub parse {
                     infile => $pp->{infile},
                 };
                 $pp->{version} = "undef";
-                $dont_delete = 1;
             } elsif ($err->{openerr}) {
+                $pp->{version} = "undef";
                 $self->_verbose(1,
                               qq{Parse::PMFile was not able to
         read the file. It issued the following error: C< $err->{r} >},
@@ -112,6 +113,7 @@ sub parse {
                     infile => $pp->{infile},
                 };
             } else {
+                $pp->{version} = "undef";
                 $self->_verbose(1, 
                               qq{Parse::PMFile was not able to
         parse the following line in that file: C< $err->{line} >
@@ -129,10 +131,6 @@ sub parse {
                     infile => $err->{file},
                 };
             }
-            unless ($dont_delete) {
-                delete $ppp->{$package};
-                next;
-            }
         }
 
         # Sanity checks
@@ -226,9 +224,10 @@ sub _parse_version {
                 if (ref $err) {
                     if ($err->{line} =~ /([\$*])([\w\:\']*)\bVERSION\b.*?\=(.*)/) {
                         local($^W) = 0;
-                        $self->_restore_overloaded_stuff if version->isa('version::vpp');
-                        $v = ($self->{UNSAFE} || $UNSAFE) ? eval $3 : $comp->reval($3);
-                        $v = $$v if $1 eq '*' && ref $v;
+                        my ($sigil, $vstr) = ($1, $3);
+                        $self->_restore_overloaded_stuff(1) if $err->{line} =~ /use\s+version\b|version\->|qv\(/;
+                        $v = ($self->{UNSAFE} || $UNSAFE) ? eval $vstr : $comp->reval($vstr);
+                        $v = $$v if $sigil eq '*' && ref $v;
                     }
                     if ($@ or !$v) {
                         $self->_verbose(1, sprintf("reval failed: err[%s] for eval[%s]",
@@ -265,31 +264,46 @@ sub _parse_version {
 }
 
 sub _restore_overloaded_stuff {
-    my $self = shift;
+    my ($self, $used_version_in_safe) = @_;
     return if $self->{UNSAFE} || $UNSAFE;
 
     no strict 'refs';
     no warnings 'redefine';
 
     # version XS in CPAN
-    if (version->isa('version::vxs')) {
+    my $restored;
+    if ($INC{'version/vxs.pm'}) {
         *{'version::(""'} = \&version::vxs::stringify;
         *{'version::(0+'} = \&version::vxs::numify;
         *{'version::(cmp'} = \&version::vxs::VCMP;
         *{'version::(<=>'} = \&version::vxs::VCMP;
         *{'version::(bool'} = \&version::vxs::boolean;
+        $restored = 1;
+    }
     # version PP in CPAN
-    } elsif (version->isa('version::vpp')) {
+    if ($INC{'version/vpp.pm'}) {
         {
             package # hide from PAUSE
                 charstar;
             overload->import;
         }
-        *{'version::(""'} = \&version::vpp::stringify;
-        *{'version::(0+'} = \&version::vpp::numify;
-        *{'version::(cmp'} = \&version::vpp::vcmp;
-        *{'version::(<=>'} = \&version::vpp::vcmp;
-        *{'version::(bool'} = \&version::vpp::vbool;
+        if (!$used_version_in_safe) {
+            package # hide from PAUSE
+                version::vpp;
+            overload->import;
+        }
+        unless ($restored) {
+            *{'version::(""'} = \&version::vpp::stringify;
+            *{'version::(0+'} = \&version::vpp::numify;
+            *{'version::(cmp'} = \&version::vpp::vcmp;
+            *{'version::(<=>'} = \&version::vpp::vcmp;
+            *{'version::(bool'} = \&version::vpp::vbool;
+        }
+        *{'version::vpp::(""'} = \&version::vpp::stringify;
+        *{'version::vpp::(0+'} = \&version::vpp::numify;
+        *{'version::vpp::(cmp'} = \&version::vpp::vcmp;
+        *{'version::vpp::(<=>'} = \&version::vpp::vcmp;
+        *{'version::vpp::(bool'} = \&version::vpp::vbool;
         *{'charstar::(""'} = \&charstar::thischar;
         *{'charstar::(0+'} = \&charstar::thischar;
         *{'charstar::(++'} = \&charstar::increment;
@@ -301,8 +315,10 @@ sub _restore_overloaded_stuff {
         *{'charstar::(<=>'} = \&charstar::spaceship;
         *{'charstar::(bool'} = \&charstar::thischar;
         *{'charstar::(='} = \&charstar::clone;
+        $restored = 1;
+    }
     # version in core
-    } else {
+    if (!$restored) {
         *{'version::(""'} = \&version::stringify;
         *{'version::(0+'} = \&version::numify;
         *{'version::(cmp'} = \&version::vcmp;
@@ -320,7 +336,6 @@ sub _packages_per_pmfile {
     my $filemtime = $self->{MTIME};
     my $version = $self->{VERSION};
 
-    $DB::single++;
     open my $fh, "<", "$pmfile" or return $ppp;
 
     local $/ = "\n";
@@ -474,7 +489,7 @@ sub _packages_per_pmfile {
             };
             local $^W = 0;
             local $SIG{__WARN__} = sub {};
-            $result = eval($eval);
+            $result = __clean_eval($eval);
             # warn "current_parsed_line[$current_parsed_line]\$\@[$@]";
             if ($@ or !defined $result){
                 die +{
@@ -489,6 +504,9 @@ sub _packages_per_pmfile {
         close FH;
 
         $result = "undef" unless defined $result;
+        if ((ref $result) =~ /^version(?:::vpp)?\b/) {
+            $result = $result->numify;
+        }
         return $result;
     }
 }
